/*
 *  Copyright (C) 2000  Henk de Groot - PE1DNN
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Please note that the GPL allows you to use the driver, NOT the radio.
 *  In order to use the radio, you need a license from the communications
 *  authority of your country.
 */

#ifndef _DIGITYPE_H_
#define _DIGITYPE_H_

/*
 * This header-file contains all the type definitions used in DIGI_NED
 *
 * Relies on the DIGICONS.H file with constants
 */

/*
 * Simple types
 */

/*
 * Enumeration types
 */
typedef enum { NONE, ADD, REPLACE, NEW, SWAP, HIJACK, ERASE } operation_t;
typedef enum { DIGIPEAT, DIGIEND, DIGITO } command_t;
typedef enum { NOACK, ACK, SHUTDOWN } ack_t;
typedef enum { ACTIVE, CLOSING_DOWN } operational_state_t;
typedef enum { LOCAL, REMOTE } distance_t;
typedef enum { EXCL_ALLBUT, INCL_ALLBUT } porttype_t;
typedef enum { NOT_KEPT, KEPT } preempt_keep_flag_t;

/*
 * Structure types
 */

/* struct for storing raw assembled RX and TX packets */
typedef struct frame_s {
    short              len;                     /* length of the frame       */
    char               port;                    /* port number               */
    unsigned char      frame[MAX_FRAME_LENGTH]; /* L1 frame without CRC      */
} frame_t;

/* struct for storing RX and TX disassembled packets */
typedef struct uidata_s {
    short              port;                    /* port of this frame        */
    distance_t         distance;                /* LOCAL or REMOTE RX and TX */
    char               primitive;               /* primitive + P/F bit       */
    unsigned short     pid;                     /* PID or 0xffff if no PID   */
    char               originator[CALL_LENGTH]; /* source call of the frame  */
    char               orig_flags;              /* flags surrounding SSID    */
    char               destination[CALL_LENGTH];/* destination call of frame */
    char               dest_flags;              /* flags surrounding SSID    */
    short              ndigi;                   /* number of digipeaters     */
    char               digipeater[MAX_DIGI][CALL_LENGTH]; /* digi calls      */
    char               digi_flags[MAX_DIGI];    /* flags surrounding SSID    */
    short              size;                    /* size of data in frame     */
    char               data[MAX_DATA_LENGTH];   /* data of the frame         */
} uidata_t;

/* struct for timer functions */
typedef struct timer_s {
    unsigned long      last_update;             /* last time or update       */
    signed short       time_to_go;              /* time before elapse in sec */
} timer_t;

/* struct for storing digipeat, digiend and digito rules */
typedef struct digipeat_s {
    struct digipeat_s *next;                    /* pointer to next rule      */
    char               digi[CALL_LENGTH];       /* digi name to act on       */
    char               output[OUTPORTS_LENGTH]; /* output ports of this rule */
    operation_t        operation;               /* which operation to apply  */
    short              use;                     /* nr. of digi-calls to use  */
    short              ssid;                 /* replacement SSID for digito: */
    char              *mapping;                 /* digis to add/substitute   */
} digipeat_t;

/* struct for storing preempt rules */
typedef struct preempt_s {
    struct preempt_s  *next;                    /* pointer to next rule      */
    char               digi[CALL_LENGTH];       /* digi name to act on       */
    char               outd[CALL_LENGTH];       /* digi after preempting     */
} preempt_t;

typedef struct preempt_keep_s {
    struct preempt_keep_s  *next;               /* pointer to next rule      */
    char                    digi[CALL_LENGTH];  /* digi name to keep         */
    preempt_keep_flag_t     keep;               /* mark which digis are kept */
} preempt_keep_t;

/* struct for storing message_paths for messages */
typedef struct message_path_s {
    struct message_path_s *next;                /* pointer to next parh      */
    char                  *path;                /* digipeat path             */
} message_path_t;

/* struct for storing messages for transmission */
typedef struct message_s {
    struct message_s  *next;                    /* pointer to next message   */
    timer_t            timer;                   /* timer to start next try   */
    short              interval;                /* time between tries        */
    short              count;                   /* repeat count if not acked */
    short              port;                    /* port to transmit on       */
    char               id;                      /* message id or -1 for none */
    char               message[MSG_PREFIX_LENGTH + MAX_MESSAGE_LENGTH 
                                + MSG_SUFFIX_LENGTH + 1];
                                                /* message + space for '\0'  */
} message_t;

/* struct for storing data for the exit command */
typedef struct exit_info_s {
    operational_state_t state;                  /* flag for closing down     */
    short               id;                     /* id of shutdown message    */
    short               exitcode;               /* exitcode when exiting     */
} exit_info_t;

/* struct for storing send (beacon) messages */
typedef struct send_s {
    struct send_s     *next;                    /* pointer to next beacon    */
    timer_t            timer;                   /* timer to go for beacon    */
    short              interval;                /* time between beacons      */
    char               output[OUTPORTS_LENGTH]; /* output ports of this rule */
    char              *path;                    /* digipeat path             */
    char              *file;                    /* file containing text      */
} send_t;

/* struct for keeping information about handled data */
typedef struct keep_s {
    struct keep_s     *next;                    /* pointer to next keep data */
    timer_t            keep_timer;              /* time before removal       */
    char               originator[CALL_LENGTH]; /* source call of the frame  */
    unsigned short     crc;                     /* 16 bit CRC of data        */
} keep_t;

/* struct for keeping calls in lists for various purposes */
typedef struct call_s {
    struct call_s     *next;                    /* pointer to next call      */
    char               call[CALL_LENGTH];       /* the call which is kept    */
} call_t;

/* struct for keeping heard direct calls*/
typedef struct mheard_s {
    struct mheard_s   *next;                    /* pointer to next block call*/
    char               call[CALL_LENGTH];       /* call which was heard      */
    long               when;                    /* when the call was heard   */
    short              port;                    /* port on which it is heard */
} mheard_t;

typedef struct query_s {
    struct query_s *next;                       /* pointer to next query     */
    char   query[MAX_MESSAGE_LENGTH];           /* the query                 */
    char   answer[MAX_MESSAGE_LENGTH];          /* the answer                */
} query_t;

#endif /* _DIGITYPE_H_ */
