/*
 *  Copyright (C) 2000  Henk de Groot - PE1DNN
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Please note that the GPL allows you to use the driver, NOT the radio.
 *  In order to use the radio, you need a license from the communications
 *  authority of your country.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#ifndef _LINUX_
#include <conio.h>
#include <dos.h>
#include <direct.h>
#include <dir.h>
#else
#include <signal.h>
#include "linux.h"
#endif /* _LINUX_ */
#include "main.h"
#include "digicons.h"
#include "digitype.h"
#include "genlib.h"
#include "mac_if.h"
#include "read_ini.h"
#include "digipeat.h"
#include "send.h"
#include "message.h"
#include "output.h"
#include "mheard.h"

static char  *progname;
static char  *progpath;

#ifdef _LINUX_
void leave(int sig)
{
    fflush(stdout);
    exit(0);
}
#else
void dos_shell(void)
{
    char    *p;
    frame_t  frame;
    char     cwd[MAXPATH + 1];
    short    disk;

    say("Shell to DOS, type exit to return to DIGI_NED\r\n");
    display_separator();

    getcwd(cwd, MAXPATH);
    disk = _getdrive();

    p = getenv("COMSPEC");

    system(p);

    /* change back to our own directory */
    chdir(cwd);
    _chdrive(disk);

    /*
     * flush pending input, we could have been away
     * for a long time, don't transmit old data
     */
    while(mac_avl())
    {
        /* flush pending input */
        (void) mac_inp(&frame);
    }

    say("Welcome back to DIGI_NED, running\r\n");
    display_separator();
}
#endif /* _LINUX_ */

char *get_progpath(void)
{
    return progpath;
}

/*
 * Entry point of DIGI_NED. First parse parameters, then enter the
 * program-loop.
 */
int main(int argc, char *argv[])
{
    uidata_t    uidata;
    short       result;
    short       ports;
    short       errflg;
    char       *p,*q;
    short       optind;
    short       c;
    short       verbose;
#ifndef _LINUX_
    char        kb_command[MAX_MESSAGE_LENGTH + 1];
    char        kb_pos;

    /* index in keyboard command array to 0 */
    kb_pos = 0;
#endif /* _LINUX_ */

    /* wether or not to use direct video access */
    directvideo = 1;

    /* assume no errors */
    errflg = 0;

    /* number of ports to 0 */
    ports = 0;

    /* verbose flag used for startup message */
    verbose = 0;

    /* remember the path to the executable and the executable name */
    progpath = strdup(argv[0]);

    if(progpath == NULL)
    {
        fprintf(stderr, "Out of memory while copying program-path\n");
        return -1;
    }

    p = strrchr(progpath, (int) '/');

    if(p == NULL)
    {
        /* no forward slashes in name, maybe backward slashes? */
        p = strrchr(progpath, (int) '\\');
        if(p == NULL)
        {
            /* no path */
            progname = progpath;
            progpath = "";
        }
        else
        {
            p++;
            progname = strdup(p);
            if(progname == NULL)
            {
                fprintf(stderr, "Out of memory while copying program-name\n");
                return -1;
            }

            /* patch end of path to '\0' */
            *p = '\0';
        }
    }
    else
    {
        p++;
        progname = strdup(p);
        if(progname == NULL)
        {
            fprintf(stderr, "Out of memory while copying program-name\n");
            return -1;
        }

        /* patch end of path to '\0' */
        *p = '\0';
    }

    /*
     * I wanted to use 'getopt' but that function is not available in
     * BCC3.1
     */
    optind = 1;
    while( (optind < argc)
           && 
           (
             (argv[optind][0] == '-')
             ||
             (argv[optind][0] == '/')
           )
           && 
           (argv[optind][1] != '\0')
           && 
           (argv[optind][2] == '\0')
         )
    {
        c = argv[optind][1];

        switch (c) {
#ifdef _LINUX_
        case 'P':
        case 'p':
                    if((optind + 1) != argc)
                    {
                        optind++;

                        if(add_lnx_port(argv[optind]) == 1)
                        {
                            ports++;
                        }
                        else
                        {
                            fprintf(stderr, "Too many ports specified.\n");
                            errflg++;
                        }
                    }
                    else
                    {
                        fprintf(stderr, "Option '-p' missing a port\n");
                    }
                    break;
#endif /* _LINUX_ */
        case 'V':
        case 'v':
                    set_verbose();
                    verbose = 1;
                    break;
        case 'H':
        case 'h':
                    display_help();
                    return 0;
        default:
                    fprintf(stderr, "Invalid argument on commandline.\n");
                    errflg++;
                    break;
        }
        optind++;
    }
    if((optind + 1) < argc)
    {
        /*
         * more than 1 option still not parsed, there van only be one
         * option still left over at this time and that is the optional
         * specification of the .ini file. Now there are more than one,
         * which is wrong.
         */
        fprintf(stderr, "Commandline error.\n");
        errflg++;
    }

#ifndef _LINUX_
    if (errflg != 0)
    {
        fprintf(stderr, "Usage: %s [-h] | [-v] [<digi_ned.ini>]\n",
                         progname);
        return -1;
    }

    if(init_mac() == 0)
    {
        fprintf(stderr,"Can't find AX25_MAC in memory!\n");
        return -1;
    }
    else
    {
        vsay("AX25_MAC found at interrupt vector 0x%02x\r\n", mac_vector);
    }

    ports = mac_ports();

    vsay("AX25_MAC has %d active ports\r\n", ports);
#else
    if ((errflg != 0) || (ports == 0))
    {
        fprintf(stderr, "Usage: %s [-h] | -p <port> [-p <port>...] [-v] "
                        "[<digi_ned.ini>]\n", progname);
        return -1;
    }

    if(init_mac() == 0)
    {
        fprintf(stderr,"Not all AX25 ports seem to be available!\n");
        return -1;
    }
    else
    {
        vsay("AX25 ports initialized\r\n");
    }

    vsay("Using %d active ports\r\n", ports);
#endif /* _LINUX_ */

    if (argc == (optind + 1))
    {
        p = strdup(argv[optind]);
        if(p == NULL)
        {
            fprintf(stderr, "Out of memory while assembling path "
                            "to the .ini file\n");
            return -1;
        }
    }
    else
    {
        p = strdup("digi_ned.ini");
        if(p == NULL)
        {
            fprintf(stderr, "Out of memory while assembling path "
                            "to the .ini file\n");
            return -1;
        }
    }

    /* see if we have to add progpath to the filename */
    if( (strrchr(p, (int) '/') == NULL)
        &&
        (strrchr(p, (int) '\\') == NULL)
        &&
        (strrchr(p, (int) ':') == NULL)
      )
    {
        /* no path in name, add progpath in front  */
        q = malloc(strlen(get_progpath()) + strlen(p) + 1);
        if(q == NULL)
        {
            fprintf(stderr, "Out of memory while assembling path "
                            "to the .ini file\n");
            return -1;
        }
        strcpy(q, get_progpath());
        strcat(q, p);
        free(p);
        p = q;
    }

    /* read the INI file */
    result = read_ini(p, ports);

    /* .ini filename not longer needed */
    free(p);

    if(result != 1)
    {
        fprintf(stderr,"Missing mandatory settings in .ini file\r\n");
        return -1;
    }

    /* initialize message system */
    init_message();

#ifndef _LINUX_
    /* program loop, exit with ALT-X */

    if(verbose == 0)
    {
        say("DIGI_NED running in non-verbose mode, use ALT-X to exit\r\n");
    }
    else
    {
        say("DIGI_NED running, use ALT-X to exit\r\n");
    }
#else
    /* program loop, exit on SIGINT signal
     * nothing special is done, termination of a program is the default
     * reaction to SIGINT
     */
    if(verbose == 0)
    {
        say("DIGI_NED running in non-verbose mode, send SIGINT signal "
                "(usualy Ctrl-C) to exit\r\n");
    }
    else
    {
        say("DIGI_NED running, send SIGINT signal (usualy Ctrl-C) to exit\r\n");
    }

    /*
     * Bail out on signals like this
     */
    signal(SIGHUP, leave);
    signal(SIGINT, leave);
    signal(SIGTERM, leave);
    signal(SIGQUIT, leave);
    signal(SIGUSR1, leave);
    signal(SIGUSR2, leave);
#endif /* _LINUX_ */

    display_separator();

    for (;;)
    {
        result = get_uidata(&uidata);

        switch(result) {
        case -1:
                exit(1);
                break;
        case 1:
                dump_uidata_from(&uidata);

                /* first check on messages */
                if(recv_message(&uidata) == 0)
                {
                    /* and then digipeat */
                    do_digipeat(&uidata);
                }

                do_mheard(&uidata);

                display_separator();
                break;
        case 2:
                /* unsuported PID, dump anyway but no further action */
                dump_uidata_from(&uidata);

                display_separator();
                break;
        case 3:
                /* corrupt frame, do nothing */

                display_separator();
                break;
        default:
                break;
        }

#ifndef _LINUX_
        if (kbhit ())
        {
            c = getch();

            if(c == 0)
            {
                /* ALT */
                if(kbhit())
                    c = getch();

                if (c == 0x2D)      /* ALT-X */
                    break;

                switch(c) {
                case 0x2f:          /* ALT-V */
                    toggle_verbose();
                    break;
                case 0x26:          /* ALT-L */
                    toggle_logging();
                    break;
                case 0x20:          /* ALT-D */
                    dos_shell();
                    break;
                default:
                    /* do nothing */
                    break;
                }
            }
            else
            {
                /* no ALT */
                if((c >= ' ') && (c < 128))
                {
                    if(kb_pos < MAX_MESSAGE_LENGTH)
                    {
                        putch(c);
                        kb_command[kb_pos] = c;
                        kb_pos++;
                    }
                }
                else
                {
                    if(c == '\r')
                    {
                        putch('\r');
                        putch('\n');
                        kb_command[kb_pos] = '\0';
                        kb_pos = 0;
                        keyboard_message(kb_command);
                        display_separator();
                    }
                    else if(c == '\b')
                    {
                        if(kb_pos > 0)
                        {
                            kb_pos--;
                            putch('\b');
                            putch(' ');
                            putch('\b');
                        }
                    }
                }
            }
        }
#endif /* _LINUX_ */

        /* handle beacon transmissions */
        do_send();

        /* handle message transmissions */
        do_messages();
    }

    fflush(stdout);
    return 0;
}
