/*
 *  Copyright (C) 2000  Henk de Groot - PE1DNN
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Please note that the GPL allows you to use the driver, NOT the radio.
 *  In order to use the radio, you need a license from the communications
 *  authority of your country.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "digicons.h"
#include "digitype.h"
#include "genlib.h"
#include "read_ini.h"
#include "mheard.h"
#include "output.h"
#ifdef _LINUX_
#include "linux.h"
#endif

mheard_t *digi_mheard_p      = NULL;
mheard_t *digi_next_mheard_p = NULL;

mheard_t *next_mheard(short port)
{
    /*
     * Implementation warning: first_mheard/next_mheard shall be used
     * without having an update the mheard list inbetween!!
     */
    mheard_t *mheard_p;

    mheard_p = digi_next_mheard_p;
    while(mheard_p != NULL)
    {
        if((mheard_p->port == port) || (port == -1))
        {
            /* bingo! found one for this port */

            /* next time start with the next data-block */
            digi_next_mheard_p = mheard_p->next;

            /* return the matching mheard pointer */
            return mheard_p;
        }
        mheard_p = mheard_p->next;
    }

    /* mark that we are at the end */
    digi_next_mheard_p = NULL;

    /* no more matches, return NULL */
    return NULL;
}

mheard_t *first_mheard(short port)
{
    /*
     * Implementation warning: first_mheard/next_mheard shall be used
     * without having an update the mheard list inbetween!!
     */

    /* reset running pointer to the start */
    digi_next_mheard_p = digi_mheard_p;

    /* simply return the first match */
    return next_mheard(port);
}

mheard_t *find_mheard(char * call)
{
    mheard_t *mheard_p;

    /* start from the start */
    mheard_p = digi_mheard_p;

    while(mheard_p != NULL)
    {
        if(stricmp(call, mheard_p->call) == 0)
        {
            /* found! */
            return mheard_p;
        }

        mheard_p = mheard_p->next;
    }

    return NULL;
}

static mheard_t *lookup_mheard(char *call)
{
    mheard_t *mheard_p;
    mheard_t *next_mheard_p;
    mheard_t *previous_mheard_p;
    short     count;

    /*
     * Loop though mheards to see if the call is present already
     *
     * If the call is present then unlink its data and return
     * the pointer to the data to the caller.
     */
    count             = 0;
    mheard_p          = digi_mheard_p;
    previous_mheard_p = NULL;
    while(mheard_p != NULL)
    {
        /* count the entries */
        count++;

        /* save next */
        next_mheard_p = mheard_p->next;
        
        if( (stricmp(call, mheard_p->call) == 0)
            ||
            (count == digi_heard_size)
          )
        {
            /* unlink this mheard data from the list */
            if(previous_mheard_p == NULL)
            {
                /* still at the beginning of the list */
                digi_mheard_p = next_mheard_p;
            }
            else
            {
                /* not at the beginning of the list */
                previous_mheard_p->next = next_mheard_p;
            }

            /* 
             * The mheard data for the call, or the last entry when the
             * list is full, is not in the list anymore, return for use.
             */
            return mheard_p;
        }
        else
        {
            /* keep this mheard */
            previous_mheard_p = mheard_p;
            mheard_p = next_mheard_p;
        }
    }
    /* 
     * If we end up here then the call is not in the heard-list yet and
     * the list is not full (otherwise the oldest mheard data block is
     * returned for reuse). Return a fresh data block or NULL when out
     * of memory.
     */
    return malloc(sizeof(mheard_t));
}

void do_mheard(uidata_t *uidata_p)
{
    mheard_t *mheard_p;

    /* bail out if not received directly */
    if(uidata_p->distance != LOCAL)
    {
        return;
    }

    /* the frame was received directly, get an entry from the mheard list */
    mheard_p = lookup_mheard(uidata_p->originator);

    if(mheard_p == NULL)
    {
        vsay("Out of memory on mheard handling\r\n");
        return;
    }

    /* fill in our data */
    strcpy(mheard_p->call, uidata_p->originator);
    mheard_p->port = uidata_p->port;
    mheard_p->when = (long) time(NULL);

    /* add to the front of the list */
    mheard_p->next = digi_mheard_p;
    digi_mheard_p  = mheard_p;

    /* finished */
    return;
}
