/*
 *  Copyright (C) 2000  Henk de Groot - PE1DNN
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Please note that the GPL allows you to use the driver, NOT the radio.
 *  In order to use the radio, you need a license from the communications
 *  authority of your country.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "main.h"
#include "digicons.h"
#include "digitype.h"
#include "genlib.h"
#include "read_ini.h"
#include "digipeat.h"
#include "send.h"
#include "call.h"
#include "output.h"
#include "message.h"
#include "preempt.h"
#ifdef _LINUX_
#include "linux.h"
#endif

short        digi_heard_size;
short        digi_keep_time;
short        digi_short_keep_time;
short        digi_message_keep_time;
short        digi_enable_exit;
short        digi_max_msg_hops;
char         digi_digi_call[10];
char         digi_digi_dest[10];
char        *digi_data_prefix;
char        *digi_logfile = ""; /* needs to initialized at startup */
char        *digi_message_file;

short        linecount;
static short ini_ports;
static FILE *ini_fd;

void process_ini_line(char *line)
{
    char *token;
    char *value;
    char *tmp;

    /* get token */
    token = strtok(line," \t:\r\n");

    /* check if we got a token */
    if(token == NULL)
        return;

    /* if token starts with ';' then it is a comment-line */
    if(token[0] == ';')
        return;

    /* convert to lowercase */
    strlwr(token);

    if(strcmp(token, "send") == 0)
    {
        add_send(ini_fd, ini_ports);
        return;
    }

    if(strcmp(token, "digipeat") == 0)
    {
        add_digipeat(ini_ports);
        return;
    }

    if(strcmp(token, "digiend") == 0)
    {
        add_digiend(ini_ports);
        return;
    }

    if(strcmp(token, "digito") == 0)
    {
        add_digito(ini_ports);
        return;
    }

    if(strcmp(token, "local") == 0)
    {
        add_local(ini_ports);
        return;
    }

    if(strcmp(token, "block") == 0)
    {
        add_block();
        return;
    }

    if(strcmp(token, "msg_block") == 0)
    {
        add_msg_block();
        return;
    }

    if(strcmp(token, "digi_owner") == 0)
    {
        add_owner();
        return;
    }

    if(strcmp(token, "message_path") == 0)
    {
        add_message_path(ini_ports);
        return;
    }

    if(strcmp(token, "preempt") == 0)
    {
        add_preempt(ini_ports);
        return;
    }

    if(strcmp(token, "preempt_keep") == 0)
    {
        add_preempt_keep();
        return;
    }

    if(strcmp(token, "preempt_never_keep") == 0)
    {
        add_preempt_never_keep();
        return;
    }

    value = strtok(NULL," \t\r\n");
    if(value == NULL)
    {
        /* if there is no data anymore, use an empty string */
        value = "";
    }

    if(strcmp(token, "size_heard_list") == 0)
    {
        digi_heard_size = atoi(value);
        return;
    }

    if(strcmp(token, "keep_time") == 0)
    {
        digi_keep_time = atoi(value);
        return;
    }

    if(strcmp(token, "short_keep_time") == 0)
    {
        digi_short_keep_time = atoi(value);
        return;
    }

    if(strcmp(token, "message_keep_time") == 0)
    {
        digi_message_keep_time = atoi(value);
        return;
    }

    if(strcmp(token, "enable_exit") == 0)
    {
        digi_enable_exit = atoi(value);
        return;
    }

    if(strcmp(token, "max_msg_hops") == 0)
    {
        digi_max_msg_hops = atoi(value);
        return;
    }

    if(strcmp(token, "data_prefix") == 0)
    {
        free(digi_data_prefix);
        digi_data_prefix = strdup(value);
        return;
    }

    if(strcmp(token, "logfile") == 0)
    {
        free(digi_logfile);

        /* slash handling */
        tmp = value;
        while(*tmp != '\0')
        {
#ifndef _LINUX_
            if(*tmp == '/' ) { *tmp = '\\'; }
#else
            if(*tmp == '\\') { *tmp = '/';  }
#endif /* _LINUX_ */
            tmp++;
        }

        /* see if we have to add progpath to the filename */
        if( (strrchr(value, (int) '/') == NULL)
            &&
            (strrchr(value, (int) '\\') == NULL)
            &&
            (strrchr(value, (int) ':') == NULL)
            &&
            (strlen(value) != 0)
          )
        {
            /* no path in name, add progpath in front  */
            tmp = malloc(strlen(get_progpath()) + strlen(value) + 1);
            if(tmp != NULL)
            {
                strcpy(tmp, get_progpath());
                strcat(tmp, value);
            }
            digi_logfile = tmp;
        }
        else
        {
            /* name includes path */
            digi_logfile = strdup(value);
        }
        return;
    }

    if(strcmp(token, "message_file") == 0)
    {
        free(digi_message_file);

        /* slash handling */
        tmp = value;
        while(*tmp != '\0')
        {
#ifndef _LINUX_
            if(*tmp == '/' ) { *tmp = '\\'; }
#else
            if(*tmp == '\\') { *tmp = '/';  }
#endif /* _LINUX_ */
            tmp++;
        }

        /* see if we have to add progpath to the filename */
        if( (strrchr(value, (int) '/') == NULL)
            &&
            (strrchr(value, (int) '\\') == NULL)
            &&
            (strrchr(value, (int) ':') == NULL)
            &&
            (strlen(value) != 0)
          )
        {
            /* no path in name, add progpath in front  */
            tmp = malloc(strlen(get_progpath()) + strlen(value) + 1);
            if(tmp != NULL)
            {
                strcpy(tmp, get_progpath());
                strcat(tmp, value);
            }
            digi_message_file = tmp;
        }
        else
        {
            /* name includes path */
            digi_message_file = strdup(value);
        }
        return;
    }

    if( (strcmp(token, "digi_call") != 0)
        &&
        (strcmp(token, "digi_dest") != 0)
      )
    {
        say("Unknown token: %s at line %d\r\n", token, linecount);
        return;
    }

    if(strlen(value) > 9)
    {
        say("Call (%s) not okay at token %s: at line %d\r\n", 
                value, token, linecount);
        return;
    }

    strupr(value); /* calls in uppercase */

    if(strcmp(token, "digi_call") == 0)
    {
        strcpy(digi_digi_call, value);
        strupr(digi_digi_call);
        return;
    }

    if(strcmp(token, "digi_dest") == 0)
    {
        strcpy(digi_digi_dest, value);
        strupr(digi_digi_dest);
        return;
    }
}

short read_ini(char *filename, short ports)
{
    char  line[256];

    vsay("Reading %s...\r\n",filename);

    digi_heard_size        = 4;   /* 4 calls = one message line response */
    digi_keep_time         = 300; /* 5 minutes */
    digi_short_keep_time   = 10;  /* 10 seconds */
    digi_message_keep_time = 900; /* 15 minutes */
    digi_enable_exit       = 0;   /* default: not enabled */
    digi_max_msg_hops      = 8;   /* maximum number of hops */
    digi_digi_call[0]      = '\0';
    digi_digi_dest[0]      = '\0';
    digi_data_prefix       = strdup("");
    digi_logfile           = strdup("");
    digi_message_file      = strdup("");

    ini_ports = ports;

    ini_fd = fopen(filename,"rt");
    if(ini_fd == NULL)
    {
        perror(filename);
        return 0;
    }
    linecount = 0;

    while(fgets(&line[0], 256, ini_fd) != NULL)
    {
        linecount++;
        process_ini_line(line);
    }

    fclose(ini_fd);

    if(digi_owner() == NULL)
    {
        say("Missing digi_owner, the callsign of the owner of the digi!\r\n");
        return 0;
    }

    if(digi_digi_call[0]  == '\0')
    {
        say("Missing digi_call, the callsign of this digi!\r\n");
        return 0;
    }

    if(digi_digi_dest[0]  == '\0')
    {
        say("Missing digi_dest, the callsign to which this digi "
               "should sends its beacons!\r\n");
        return 0;
    }

    swap_digicall_digidest_digi(ini_ports);
    swap_digicall_digidest_send();
    swap_digicall_digidest_preempt(ini_ports);

    vsay("%s Read, %d lines\r\n",filename, linecount);

    return 1;
}

