/*
 *  Copyright (C) 2000  Henk de Groot - PE1DNN
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Please note that the GPL allows you to use the driver, NOT the radio.
 *  In order to use the radio, you need a license from the communications
 *  authority of your country.
 */

#include <time.h>
#include <stdlib.h>
#include "digicons.h"
#include "digitype.h"
#include "genlib.h"
#include "timer.h"

/*
 * The timers in this module use signed short as type. This means
 * that the timers can time up-to 32767 seconds (using 1s timer ticks
 * from the time(NULL) system call).
 *
 * EXPIRED is given the maximum negative value. This is -32768, one
 * more than the maximum difference in time (so it can never occur in
 * calculating <timervalue> - difference since <timervalue> is 0 or
 * positive!
 */
#define EXPIRED ((signed short) 0x8000) /* Expired timer-value */

/*
 * The MAXTIME value is used as difference when a timer update
 * takes place more than 32767 seconds after the last-update. In that
 * case the difference is set to MAXTIME which will cause all timers
 * to expire.
 */
#define MAXTIME ((signed short) 0xffff) /* Maximum timer-value */

void start_timer(timer_t *tm, signed short timer_time)
{
    /*
     * Now this is realy simple!
     */
    tm->last_update = (unsigned long) time(NULL);
    tm->time_to_go  = timer_time;
}

void clear_timer(timer_t *tm)
{
    /*
     * Now this is realy simple!
     */
    tm->time_to_go = EXPIRED;
}

short is_elapsed(timer_t *tm)
{
    unsigned long now;
    unsigned long big_difference;
    signed short  difference;

    /*
     * Get 10ms counter from the environment.
     */
    now = (unsigned long) time(NULL);

    /*
     * Calculate the time that has passed. This relies on wapping arround
     * on overflow (for example now = 10 and last_update = 4294967290).
     * Keep the current time in last_update.
     */
    big_difference  = now - tm->last_update;
    tm->last_update = now;
    if(big_difference > ((unsigned long) MAXTIME))
    {
        difference = MAXTIME;
    }
    else
    {
        difference = (signed short) big_difference;
    }

    /*
     * If difference is 0 then there is nothing to update, just skip.
     */
    if ( ( difference != 0 )
         &&
         ( tm->time_to_go != EXPIRED )
       )
    {
        /*
         * Decrement time_togo
         */
        tm->time_to_go = tm->time_to_go - difference;

        if( (tm->time_to_go) <= 0 )
        {
            tm->time_to_go = EXPIRED;
        }
    }
    /*
     * Now see if the timer is expired
     */
    if( tm->time_to_go == EXPIRED )
    {
        return 1;
    }

    return 0;
}
